<?php

namespace Drupal\calendar_systems\Plugin\Block;

use Drupal;
use Drupal\Core\Annotation\Translation;
use Drupal\Core\Block\Annotation\Block;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\LoggerChannelTrait;
use Drupal\Core\Messenger\MessengerTrait;
use function drupal_get_user_timezone;

/**
 * Provides a block to show a localized current/relative date.
 *
 * @Block (
 *   id = "calendar_systems_block",
 *   admin_label=  @Translation("Calendar Systems Block"),
 * )
 */
class CalendarSystemsBlock extends BlockBase {

  use MessengerTrait;
  use LoggerChannelTrait;

  private const DEFAULT_CACHE_PERIOD_SECONDS = 3600;

  private function getCalConfig(): array {
    $cfg = $this->getConfiguration();
    $def = $this->defaultConfiguration();
    foreach ([
               'calendar_systems_calendar',
               'calendar_systems_format',
               'calendar_systems_timezone',
               'calendar_systems_text',
             ] as $key) {
      if (empty($cfg[$key])) {
        $cfg[$key] = $def[$key];
      }
    }
    $key = 'cache';
    if (!array_key_exists($key, $cfg)) {
      $cfg[$key] = $def[$key];
    }
    return $cfg;
  }

  /**
   * {@inheritDoc}
   */
  public function defaultConfiguration(): array {
    return [
      'calendar_systems_calendar' => 'global',
      'calendar_systems_format' => 'Y/m/d H:i:s',
      'calendar_systems_timezone' => 'user',
      'calendar_systems_text' => '{}',
      'cache' => static::DEFAULT_CACHE_PERIOD_SECONDS,
    ];
  }

  /**
   * {@inheritDoc}
   */
  public function blockForm($form, FormStateInterface $form_state): array {
    $f = parent::blockForm($form, $form_state);

    $cfg = $this->getCalConfig();

    $f['calendar_systems_calendar'] = [
      '#title' => t('Calendar'),
      '#type' => 'select',
      '#options' => [
        'persian' => t('Persian'),
        'gregorian' => t('Gregorian'),
        'global' => t("Global (by site's language"),
      ],
      '#default_value' => $cfg['calendar_systems_calendar'],
    ];
    $f['calendar_systems_format'] = [
      '#title' => t('Date/Time format'),
      '#type' => 'textfield',
      '#description' => 'TODO add medium short and ... See php.net/manual/en/function.date.php for date formats',
      '#default_value' => $cfg['calendar_systems_format'],
    ];
    $f['calendar_systems_timezone'] = [
      '#title' => t('Timezone'),
      '#type' => 'select',
      '#options' => [
          'site' => t("Use site's timezone"),
          'user' => t("Use user's timezone"),
        ] + system_time_zones(),
      '#default_value' => $cfg['calendar_systems_timezone'],
    ];
    $f['calendar_systems_text'] = [
      '#title' => t('Content'),
      '#type' => 'textfield',
      '#description' => t('The blocks content. {} is replaced with the actual date. If unsure, leave this field empty or set it to {}'),
      '#default_value' => $cfg['calendar_systems_text'],
    ];
    $f['cache'] = [
      '#title' => t('Cache lifetime'),
      '#type' => 'textfield',
      '#description' => t('How long the block should be cached, in seconds'),
      '#default_value' => $cfg['cache'],
    ];

    return $f;
  }

  /**
   * {@inheritDoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->setConfigurationValue('calendar_systems_format', $form_state->getValue('calendar_systems_format'));
    $this->setConfigurationValue('calendar_systems_calendar', $form_state->getValue('calendar_systems_calendar'));
    $this->setConfigurationValue('calendar_systems_timezone', $form_state->getValue('calendar_systems_timezone'));

    if (strpos($form_state->getValue('calendar_systems_text'), '{}') < 0) {
      $this->messenger()->addWarning(t('Invalid content, content set to {}'));
      $this->setConfigurationValue('calendar_systems_text', '{}');
    }
    else {
      $this->setConfigurationValue('calendar_systems_text', $form_state->getValue('calendar_systems_text'));
    }

    $c = $form_state->getValue('cache');
    if ($c !== '' && is_numeric($c) && $c >= 0) {
      $this->setConfigurationValue('cache', $form_state->getValue('cache'));
    }
    else {
      $this->messenger()->addWarning(t('block cache time set to one hour'));
      $this->setConfigurationValue('cache', 3600);
    }
  }

  /**
   * {@inheritDoc}
   */
  public function getCacheMaxAge() {
    return $this->getCalConfig()['cache'];
  }

  /**
   * {@inheritDoc}
   */
  public function build(): array {
    $cfg = $this->getCalConfig();

    if ($cfg['calendar_systems_calendar'] === 'global') {
      $cfg['calendar_systems_calendar'] = '';
    }

    switch ($cfg['calendar_systems_timezone']) {
      case 'site':
        $config = Drupal::config('system.date');
        $config_data_default_timezone = $config->get('timezone.default');
        $tz = !empty($config_data_default_timezone) ? $config_data_default_timezone : @date_default_timezone_get();
        break;
      case 'user':
        $tz = drupal_get_user_timezone();
        break;
      default:
        $tz = $cfg['calendar_systems_timezone'];
    }

    $calendar = calendar_systems_factory($tz, NULL, $cfg['calendar_systems_calendar']);
    if (!$calendar) {
      $this->getLogger(get_class($this))
        ->error('could not construct calendar for block=%b / %bb', [
          '%b' => $this->getBaseId(),
          '%bb' => $this->label(),
        ]);
    }

    $content = $calendar
      ? str_replace('{}', $calendar->format($cfg['calendar_systems_format']), $cfg['calendar_systems_text'])
      : '';

    return ['#markup' => $content];
  }

}
